"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AwsRekognition = void 0;
const GenericFunctions_1 = require("./GenericFunctions");
class AwsRekognition {
    constructor() {
        this.description = {
            displayName: 'AWS Rekognition',
            name: 'awsRekognition',
            icon: 'file:rekognition.svg',
            group: ['output'],
            version: 1,
            subtitle: '={{$parameter["operation"] + ": " + $parameter["resource"]}}',
            description: 'Sends data to AWS Rekognition',
            defaults: {
                name: 'AWS Rekognition',
                color: '#305b94',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'aws',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    options: [
                        {
                            name: 'Image',
                            value: 'image',
                        },
                    ],
                    default: 'image',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    options: [
                        {
                            name: 'Analyze',
                            value: 'analyze',
                        },
                    ],
                    default: 'analyze',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Type',
                    name: 'type',
                    type: 'options',
                    options: [
                        {
                            name: 'Detect Faces',
                            value: 'detectFaces',
                        },
                        {
                            name: 'Detect Labels',
                            value: 'detectLabels',
                        },
                        {
                            name: 'Detect Moderation Labels',
                            value: 'detectModerationLabels',
                        },
                        {
                            name: 'Recognize Celebrity',
                            value: 'recognizeCelebrity',
                        },
                    ],
                    default: 'detectFaces',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Binary Data',
                    name: 'binaryData',
                    type: 'boolean',
                    default: false,
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'analyze',
                            ],
                            resource: [
                                'image',
                            ],
                        },
                    },
                    description: 'If the image to analize should be taken from binary field.',
                },
                {
                    displayName: 'Binary Property',
                    displayOptions: {
                        show: {
                            operation: [
                                'analyze',
                            ],
                            resource: [
                                'image',
                            ],
                            binaryData: [
                                true,
                            ],
                        },
                    },
                    name: 'binaryPropertyName',
                    type: 'string',
                    default: 'data',
                    description: 'Object property name which holds binary data.',
                    required: true,
                },
                {
                    displayName: 'Bucket',
                    name: 'bucket',
                    displayOptions: {
                        show: {
                            operation: [
                                'analyze',
                            ],
                            resource: [
                                'image',
                            ],
                            binaryData: [
                                false,
                            ],
                        },
                    },
                    type: 'string',
                    default: '',
                    required: true,
                    description: 'Name of the S3 bucket',
                },
                {
                    displayName: 'Name',
                    name: 'name',
                    displayOptions: {
                        show: {
                            operation: [
                                'analyze',
                            ],
                            resource: [
                                'image',
                            ],
                            binaryData: [
                                false,
                            ],
                        },
                    },
                    type: 'string',
                    default: '',
                    required: true,
                    description: 'S3 object key name',
                },
                {
                    displayName: 'Additional Fields',
                    name: 'additionalFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'analyze',
                            ],
                            resource: [
                                'image',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Version',
                            name: 'version',
                            displayOptions: {
                                show: {
                                    '/binaryData': [
                                        false,
                                    ],
                                },
                            },
                            type: 'string',
                            default: '',
                            description: 'If the bucket is versioning enabled, you can specify the object version',
                        },
                        {
                            displayName: 'Max Labels',
                            name: 'maxLabels',
                            type: 'number',
                            displayOptions: {
                                show: {
                                    '/type': [
                                        'detectModerationLabels',
                                        'detectLabels',
                                    ],
                                },
                            },
                            default: 0,
                            typeOptions: {
                                minValue: 0,
                            },
                            description: `Maximum number of labels you want the service to return in the response. The service returns the specified number of highest confidence labels.`,
                        },
                        {
                            displayName: 'Min Confidence',
                            name: 'minConfidence',
                            type: 'number',
                            displayOptions: {
                                show: {
                                    '/type': [
                                        'detectModerationLabels',
                                        'detectLabels',
                                    ],
                                },
                            },
                            default: 0,
                            typeOptions: {
                                minValue: 0,
                                maxValue: 100,
                            },
                            description: `Specifies the minimum confidence level for the labels to return. Amazon Rekognition doesn't return any labels with a confidence level lower than this specified value.`,
                        },
                        {
                            displayName: 'Attributes',
                            name: 'attributes',
                            type: 'multiOptions',
                            displayOptions: {
                                show: {
                                    '/type': [
                                        'detectFaces',
                                    ],
                                },
                            },
                            options: [
                                {
                                    name: 'All',
                                    value: 'all',
                                },
                                {
                                    name: 'Default',
                                    value: 'default',
                                },
                            ],
                            default: [],
                            description: `An array of facial attributes you want to be returned`,
                        },
                    ],
                },
            ],
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnData = [];
        const qs = {};
        let responseData;
        const resource = this.getNodeParameter('resource', 0);
        const operation = this.getNodeParameter('operation', 0);
        for (let i = 0; i < items.length; i++) {
            if (resource === 'image') {
                if (operation === 'analyze') {
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    let action = undefined;
                    let body = {};
                    const type = this.getNodeParameter('type', 0);
                    if (type === 'detectModerationLabels') {
                        action = 'RekognitionService.DetectModerationLabels';
                        if (additionalFields.minConfidence) {
                            body['MinConfidence'] = additionalFields.minConfidence;
                        }
                    }
                    if (type === 'detectFaces') {
                        action = 'RekognitionService.DetectFaces';
                        if (additionalFields.attributes) {
                            body['Attributes'] = additionalFields.attributes;
                        }
                    }
                    if (type === 'detectLabels') {
                        action = 'RekognitionService.DetectLabels';
                        if (additionalFields.minConfidence) {
                            body['MinConfidence'] = additionalFields.minConfidence;
                        }
                        if (additionalFields.maxLabels) {
                            body['MaxLabels'] = additionalFields.maxLabels;
                        }
                    }
                    if (type === 'recognizeCelebrity') {
                        action = 'RekognitionService.RecognizeCelebrities';
                    }
                    const binaryData = this.getNodeParameter('binaryData', 0);
                    if (binaryData) {
                        const binaryPropertyName = this.getNodeParameter('binaryPropertyName', 0);
                        if (items[i].binary === undefined) {
                            throw new Error('No binary data exists on item!');
                        }
                        if (items[i].binary[binaryPropertyName] === undefined) {
                            throw new Error(`No binary data property "${binaryPropertyName}" does not exists on item!`);
                        }
                        const binaryPropertyData = items[i].binary[binaryPropertyName];
                        body = {
                            Image: {
                                Bytes: binaryPropertyData.data,
                            },
                        };
                    }
                    else {
                        const bucket = this.getNodeParameter('bucket', i);
                        const name = this.getNodeParameter('name', i);
                        body = {
                            Image: {
                                S3Object: {
                                    Bucket: bucket,
                                    Name: name,
                                },
                            },
                        };
                        if (additionalFields.version) {
                            body.Image.S3Object.Version = additionalFields.version;
                        }
                    }
                    responseData = await GenericFunctions_1.awsApiRequestREST.call(this, 'rekognition', 'POST', '', JSON.stringify(body), {}, { 'X-Amz-Target': action, 'Content-Type': 'application/x-amz-json-1.1' });
                }
            }
            if (Array.isArray(responseData)) {
                returnData.push.apply(returnData, responseData);
            }
            else {
                returnData.push(responseData);
            }
        }
        return [this.helpers.returnJsonArray(returnData)];
    }
}
exports.AwsRekognition = AwsRekognition;
//# sourceMappingURL=AwsRekognition.node.js.map