"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Crypto = void 0;
const lodash_1 = require("lodash");
const crypto_1 = require("crypto");
class Crypto {
    constructor() {
        this.description = {
            displayName: 'Crypto',
            name: 'crypto',
            icon: 'fa:key',
            group: ['transform'],
            version: 1,
            subtitle: '={{$parameter["action"]}}',
            description: 'Provide cryptographic utilities',
            defaults: {
                name: 'Crypto',
                color: '#408000',
            },
            inputs: ['main'],
            outputs: ['main'],
            properties: [
                {
                    displayName: 'Action',
                    name: 'action',
                    type: 'options',
                    options: [
                        {
                            name: 'Hash',
                            description: 'Hash a text in a specified format.',
                            value: 'hash',
                        },
                        {
                            name: 'Hmac',
                            description: 'Hmac a text in a specified format.',
                            value: 'hmac',
                        },
                        {
                            name: 'Sign',
                            description: 'Sign a string using a private key.',
                            value: 'sign',
                        },
                    ],
                    default: 'hash',
                },
                {
                    displayName: 'Type',
                    name: 'type',
                    displayOptions: {
                        show: {
                            action: [
                                'hash',
                            ],
                        },
                    },
                    type: 'options',
                    options: [
                        {
                            name: 'MD5',
                            value: 'MD5',
                        },
                        {
                            name: 'SHA256',
                            value: 'SHA256',
                        },
                        {
                            name: 'SHA512',
                            value: 'SHA512',
                        },
                    ],
                    default: 'MD5',
                    description: 'The hash type to use',
                    required: true,
                },
                {
                    displayName: 'Value',
                    name: 'value',
                    displayOptions: {
                        show: {
                            action: [
                                'hash',
                            ],
                        },
                    },
                    type: 'string',
                    default: '',
                    description: 'The value that should be hashed.',
                    required: true,
                },
                {
                    displayName: 'Property Name',
                    name: 'dataPropertyName',
                    type: 'string',
                    default: 'data',
                    required: true,
                    displayOptions: {
                        show: {
                            action: [
                                'hash',
                            ],
                        },
                    },
                    description: 'Name of the property to which to write the hash.',
                },
                {
                    displayName: 'Encoding',
                    name: 'encoding',
                    displayOptions: {
                        show: {
                            action: [
                                'hash',
                            ],
                        },
                    },
                    type: 'options',
                    options: [
                        {
                            name: 'BASE64',
                            value: 'base64',
                        },
                        {
                            name: 'HEX',
                            value: 'hex',
                        },
                    ],
                    default: 'hex',
                    required: true,
                },
                {
                    displayName: 'Type',
                    name: 'type',
                    displayOptions: {
                        show: {
                            action: [
                                'hmac',
                            ],
                        },
                    },
                    type: 'options',
                    options: [
                        {
                            name: 'MD5',
                            value: 'MD5',
                        },
                        {
                            name: 'SHA256',
                            value: 'SHA256',
                        },
                        {
                            name: 'SHA512',
                            value: 'SHA512',
                        },
                    ],
                    default: 'MD5',
                    description: 'The hash type to use',
                    required: true,
                },
                {
                    displayName: 'Value',
                    name: 'value',
                    displayOptions: {
                        show: {
                            action: [
                                'hmac',
                            ],
                        },
                    },
                    type: 'string',
                    default: '',
                    description: 'The value of which the hmac should be created.',
                    required: true,
                },
                {
                    displayName: 'Property Name',
                    name: 'dataPropertyName',
                    type: 'string',
                    default: 'data',
                    required: true,
                    displayOptions: {
                        show: {
                            action: [
                                'hmac',
                            ],
                        },
                    },
                    description: 'Name of the property to which to write the hmac.',
                },
                {
                    displayName: 'Secret',
                    name: 'secret',
                    displayOptions: {
                        show: {
                            action: [
                                'hmac',
                            ],
                        },
                    },
                    type: 'string',
                    default: '',
                    required: true,
                },
                {
                    displayName: 'Encoding',
                    name: 'encoding',
                    displayOptions: {
                        show: {
                            action: [
                                'hmac',
                            ],
                        },
                    },
                    type: 'options',
                    options: [
                        {
                            name: 'BASE64',
                            value: 'base64',
                        },
                        {
                            name: 'HEX',
                            value: 'hex',
                        },
                    ],
                    default: 'hex',
                    required: true,
                },
                {
                    displayName: 'Value',
                    name: 'value',
                    displayOptions: {
                        show: {
                            action: [
                                'sign',
                            ],
                        },
                    },
                    type: 'string',
                    default: '',
                    description: 'The value that should be signed.',
                    required: true,
                },
                {
                    displayName: 'Property Name',
                    name: 'dataPropertyName',
                    type: 'string',
                    default: 'data',
                    required: true,
                    displayOptions: {
                        show: {
                            action: [
                                'sign',
                            ],
                        },
                    },
                    description: 'Name of the property to which to write the signed value.',
                },
                {
                    displayName: 'Algorithm',
                    name: 'algorithm',
                    displayOptions: {
                        show: {
                            action: [
                                'sign',
                            ],
                        },
                    },
                    type: 'options',
                    typeOptions: {
                        loadOptionsMethod: 'getHashes',
                    },
                    default: '',
                    required: true,
                },
                {
                    displayName: 'Encoding',
                    name: 'encoding',
                    displayOptions: {
                        show: {
                            action: [
                                'sign',
                            ],
                        },
                    },
                    type: 'options',
                    options: [
                        {
                            name: 'BASE64',
                            value: 'base64',
                        },
                        {
                            name: 'HEX',
                            value: 'hex',
                        },
                    ],
                    default: 'hex',
                    required: true,
                },
                {
                    displayName: 'Private Key',
                    name: 'privateKey',
                    displayOptions: {
                        show: {
                            action: [
                                'sign',
                            ],
                        },
                    },
                    type: 'string',
                    typeOptions: {
                        alwaysOpenEditWindow: true,
                    },
                    description: 'Private key to use when signing the string.',
                    default: '',
                    required: true,
                },
            ],
        };
        this.methods = {
            loadOptions: {
                async getHashes() {
                    const returnData = [];
                    const hashes = crypto_1.getHashes();
                    for (const hash of hashes) {
                        const hashName = hash;
                        const hashId = hash;
                        returnData.push({
                            name: hashName,
                            value: hashId,
                        });
                    }
                    return returnData;
                },
            },
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnData = [];
        const length = items.length;
        const action = this.getNodeParameter('action', 0);
        let item;
        for (let i = 0; i < length; i++) {
            item = items[i];
            const dataPropertyName = this.getNodeParameter('dataPropertyName', i);
            const value = this.getNodeParameter('value', i);
            let newValue;
            if (action === 'hash') {
                const type = this.getNodeParameter('type', i);
                const encoding = this.getNodeParameter('encoding', i);
                newValue = crypto_1.createHash(type).update(value).digest(encoding);
            }
            if (action === 'hmac') {
                const type = this.getNodeParameter('type', i);
                const secret = this.getNodeParameter('secret', i);
                const encoding = this.getNodeParameter('encoding', i);
                newValue = crypto_1.createHmac(type, secret).update(value).digest(encoding);
            }
            if (action === 'sign') {
                const algorithm = this.getNodeParameter('algorithm', i);
                const encoding = this.getNodeParameter('encoding', i);
                const privateKey = this.getNodeParameter('privateKey', i);
                const sign = crypto_1.createSign(algorithm);
                sign.write(value);
                sign.end();
                newValue = sign.sign(privateKey, encoding);
            }
            let newItem;
            if (dataPropertyName.includes('.')) {
                newItem = {
                    json: JSON.parse(JSON.stringify(item.json)),
                };
            }
            else {
                newItem = {
                    json: Object.assign({}, item.json),
                };
            }
            if (item.binary !== undefined) {
                newItem.binary = item.binary;
            }
            lodash_1.set(newItem, `json.${dataPropertyName}`, newValue);
            returnData.push(newItem);
        }
        return this.prepareOutputData(returnData);
    }
}
exports.Crypto = Crypto;
//# sourceMappingURL=Crypto.node.js.map