"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DateTime = void 0;
const moment = require("moment-timezone");
const lodash_1 = require("lodash");
class DateTime {
    constructor() {
        this.description = {
            displayName: 'Date & Time',
            name: 'dateTime',
            icon: 'fa:calendar',
            group: ['transform'],
            version: 1,
            description: 'Allows you to manipulate date and time values',
            defaults: {
                name: 'Date & Time',
                color: '#408000',
            },
            inputs: ['main'],
            outputs: ['main'],
            properties: [
                {
                    displayName: 'Action',
                    name: 'action',
                    type: 'options',
                    options: [
                        {
                            name: 'Format a Date',
                            description: 'Convert a date to a different format',
                            value: 'format',
                        },
                    ],
                    default: 'format',
                },
                {
                    displayName: 'Value',
                    name: 'value',
                    displayOptions: {
                        show: {
                            action: [
                                'format',
                            ],
                        },
                    },
                    type: 'string',
                    default: '',
                    description: 'The value that should be converted.',
                    required: true,
                },
                {
                    displayName: 'Property Name',
                    name: 'dataPropertyName',
                    type: 'string',
                    default: 'data',
                    required: true,
                    displayOptions: {
                        show: {
                            action: [
                                'format',
                            ],
                        },
                    },
                    description: 'Name of the property to which to write the converted date.',
                },
                {
                    displayName: 'Custom Format',
                    name: 'custom',
                    displayOptions: {
                        show: {
                            action: [
                                'format',
                            ],
                        },
                    },
                    type: 'boolean',
                    default: false,
                    description: 'If a predefined format should be selected or custom format entered.',
                },
                {
                    displayName: 'To Format',
                    name: 'toFormat',
                    displayOptions: {
                        show: {
                            action: [
                                'format',
                            ],
                            custom: [
                                true,
                            ],
                        },
                    },
                    type: 'string',
                    default: '',
                    placeholder: 'YYYY-MM-DD',
                    description: 'The format to convert the date to.',
                },
                {
                    displayName: 'To Format',
                    name: 'toFormat',
                    type: 'options',
                    displayOptions: {
                        show: {
                            action: [
                                'format',
                            ],
                            custom: [
                                false,
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'MM/DD/YYYY',
                            value: 'MM/DD/YYYY',
                            description: 'Example: 09/04/1986',
                        },
                        {
                            name: 'YYYY/MM/DD',
                            value: 'YYYY/MM/DD',
                            description: 'Example: 1986/04/09',
                        },
                        {
                            name: 'MMMM DD YYYY',
                            value: 'MMMM DD YYYY',
                            description: 'Example: April 09 1986',
                        },
                        {
                            name: 'MM-DD-YYYY',
                            value: 'MM-DD-YYYY',
                            description: 'Example: 09-04-1986',
                        },
                        {
                            name: 'YYYY-MM-DD',
                            value: 'YYYY-MM-DD',
                            description: 'Example: 1986-04-09',
                        },
                        {
                            name: 'Unix Timestamp',
                            value: 'X',
                            description: 'Example: 513388800.879',
                        },
                        {
                            name: 'Unix Ms Timestamp',
                            value: 'x',
                            description: 'Example: 513388800',
                        },
                    ],
                    default: 'MM/DD/YYYY',
                    description: 'The format to convert the date to.',
                },
                {
                    displayName: 'Options',
                    name: 'options',
                    displayOptions: {
                        show: {
                            action: [
                                'format',
                            ],
                        },
                    },
                    type: 'collection',
                    placeholder: 'Add Option',
                    default: {},
                    options: [
                        {
                            displayName: 'From Format',
                            name: 'fromFormat',
                            type: 'string',
                            default: '',
                            description: 'In case the input format is not recognized you can provide the format ',
                        },
                        {
                            displayName: 'From Timezone',
                            name: 'fromTimezone',
                            type: 'options',
                            typeOptions: {
                                loadOptionsMethod: 'getTimezones',
                            },
                            default: 'UTC',
                            description: 'The timezone to convert from.',
                        },
                        {
                            displayName: 'To Timezone',
                            name: 'toTimezone',
                            type: 'options',
                            typeOptions: {
                                loadOptionsMethod: 'getTimezones',
                            },
                            default: 'UTC',
                            description: 'The timezone to convert to.',
                        },
                    ],
                },
            ],
        };
        this.methods = {
            loadOptions: {
                async getTimezones() {
                    const returnData = [];
                    for (const timezone of moment.tz.names()) {
                        const timezoneName = timezone;
                        const timezoneId = timezone;
                        returnData.push({
                            name: timezoneName,
                            value: timezoneId,
                        });
                    }
                    return returnData;
                },
            },
        };
    }
    async execute() {
        const items = this.getInputData();
        const length = items.length;
        const returnData = [];
        const workflowTimezone = this.getTimezone();
        let item;
        for (let i = 0; i < length; i++) {
            const action = this.getNodeParameter('action', 0);
            item = items[i];
            if (action === 'format') {
                const currentDate = this.getNodeParameter('value', i);
                const dataPropertyName = this.getNodeParameter('dataPropertyName', i);
                const toFormat = this.getNodeParameter('toFormat', i);
                const options = this.getNodeParameter('options', i);
                let newDate;
                if (currentDate === undefined) {
                    continue;
                }
                if (options.fromFormat === undefined && !moment(currentDate).isValid()) {
                    throw new Error('The date input format could not be recognized. Please set the "From Format" field');
                }
                if (Number.isInteger(currentDate)) {
                    newDate = moment.unix(currentDate);
                }
                else {
                    if (options.fromTimezone || options.toTimezone) {
                        const fromTimezone = options.fromTimezone || workflowTimezone;
                        if (options.fromFormat) {
                            newDate = moment.tz(currentDate, options.fromFormat, fromTimezone);
                        }
                        else {
                            newDate = moment.tz(currentDate, fromTimezone);
                        }
                    }
                    else {
                        if (options.fromFormat) {
                            newDate = moment(currentDate, options.fromFormat);
                        }
                        else {
                            newDate = moment(currentDate);
                        }
                    }
                }
                if (options.toTimezone || options.fromTimezone) {
                    newDate = newDate.tz(options.toTimezone || workflowTimezone);
                }
                newDate = newDate.format(toFormat);
                let newItem;
                if (dataPropertyName.includes('.')) {
                    newItem = {
                        json: JSON.parse(JSON.stringify(item.json)),
                    };
                }
                else {
                    newItem = {
                        json: Object.assign({}, item.json),
                    };
                }
                if (item.binary !== undefined) {
                    newItem.binary = item.binary;
                }
                lodash_1.set(newItem, `json.${dataPropertyName}`, newDate);
                returnData.push(newItem);
            }
        }
        return this.prepareOutputData(returnData);
    }
}
exports.DateTime = DateTime;
//# sourceMappingURL=DateTime.node.js.map