"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Dropbox = void 0;
const n8n_core_1 = require("n8n-core");
const GenericFunctions_1 = require("./GenericFunctions");
class Dropbox {
    constructor() {
        this.description = {
            displayName: 'Dropbox',
            name: 'dropbox',
            icon: 'file:dropbox.png',
            group: ['input'],
            version: 1,
            subtitle: '={{$parameter["operation"] + ": " + $parameter["resource"]}}',
            description: 'Access data on Dropbox',
            defaults: {
                name: 'Dropbox',
                color: '#0062ff',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'dropboxApi',
                    required: true,
                    displayOptions: {
                        show: {
                            authentication: [
                                'accessToken',
                            ],
                        },
                    },
                },
                {
                    name: 'dropboxOAuth2Api',
                    required: true,
                    displayOptions: {
                        show: {
                            authentication: [
                                'oAuth2',
                            ],
                        },
                    },
                },
            ],
            properties: [
                {
                    displayName: 'Authentication',
                    name: 'authentication',
                    type: 'options',
                    options: [
                        {
                            name: 'Access Token',
                            value: 'accessToken',
                        },
                        {
                            name: 'OAuth2',
                            value: 'oAuth2',
                        },
                    ],
                    default: 'accessToken',
                    description: 'Means of authenticating with the service.',
                },
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    options: [
                        {
                            name: 'File',
                            value: 'file',
                        },
                        {
                            name: 'Folder',
                            value: 'folder',
                        },
                    ],
                    default: 'file',
                    description: 'The resource to operate on.',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'file',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Copy',
                            value: 'copy',
                            description: 'Copy a file',
                        },
                        {
                            name: 'Delete',
                            value: 'delete',
                            description: 'Delete a file',
                        },
                        {
                            name: 'Download',
                            value: 'download',
                            description: 'Download a file',
                        },
                        {
                            name: 'Move',
                            value: 'move',
                            description: 'Move a file',
                        },
                        {
                            name: 'Upload',
                            value: 'upload',
                            description: 'Upload a file',
                        },
                    ],
                    default: 'upload',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'folder',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Copy',
                            value: 'copy',
                            description: 'Copy a folder',
                        },
                        {
                            name: 'Create',
                            value: 'create',
                            description: 'Create a folder',
                        },
                        {
                            name: 'Delete',
                            value: 'delete',
                            description: 'Delete a folder',
                        },
                        {
                            name: 'List',
                            value: 'list',
                            description: 'Return the files and folders in a given folder',
                        },
                        {
                            name: 'Move',
                            value: 'move',
                            description: 'Move a folder',
                        },
                    ],
                    default: 'create',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'From Path',
                    name: 'path',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'copy',
                            ],
                            resource: [
                                'file',
                                'folder',
                            ],
                        },
                    },
                    placeholder: '/invoices/original.txt',
                    description: 'The path of file or folder to copy.',
                },
                {
                    displayName: 'To Path',
                    name: 'toPath',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'copy',
                            ],
                            resource: [
                                'file',
                                'folder',
                            ],
                        },
                    },
                    placeholder: '/invoices/copy.txt',
                    description: 'The destination path of file or folder.',
                },
                {
                    displayName: 'Delete Path',
                    name: 'path',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'delete',
                            ],
                            resource: [
                                'file',
                                'folder',
                            ],
                        },
                    },
                    placeholder: '/invoices/2019/invoice_1.pdf',
                    description: 'The path to delete. Can be a single file or a whole folder.',
                },
                {
                    displayName: 'From Path',
                    name: 'path',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'move',
                            ],
                            resource: [
                                'file',
                                'folder',
                            ],
                        },
                    },
                    placeholder: '/invoices/old_name.txt',
                    description: 'The path of file or folder to move.',
                },
                {
                    displayName: 'To Path',
                    name: 'toPath',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'move',
                            ],
                            resource: [
                                'file',
                                'folder',
                            ],
                        },
                    },
                    placeholder: '/invoices/new_name.txt',
                    description: 'The new path of file or folder.',
                },
                {
                    displayName: 'File Path',
                    name: 'path',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'download',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    placeholder: '/invoices/2019/invoice_1.pdf',
                    description: 'The file path of the file to download. Has to contain the full path.',
                },
                {
                    displayName: 'Binary Property',
                    name: 'binaryPropertyName',
                    type: 'string',
                    required: true,
                    default: 'data',
                    displayOptions: {
                        show: {
                            operation: [
                                'download',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    description: 'Name of the binary property to which to<br />write the data of the read file.',
                },
                {
                    displayName: 'File Path',
                    name: 'path',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'upload',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    placeholder: '/invoices/2019/invoice_1.pdf',
                    description: 'The file path of the file to upload. Has to contain the full path. The parent folder has to exist. Existing files get overwritten.',
                },
                {
                    displayName: 'Binary Data',
                    name: 'binaryData',
                    type: 'boolean',
                    default: false,
                    displayOptions: {
                        show: {
                            operation: [
                                'upload',
                            ],
                            resource: [
                                'file',
                            ],
                        },
                    },
                    description: 'If the data to upload should be taken from binary field.',
                },
                {
                    displayName: 'File Content',
                    name: 'fileContent',
                    type: 'string',
                    default: '',
                    displayOptions: {
                        show: {
                            operation: [
                                'upload',
                            ],
                            resource: [
                                'file',
                            ],
                            binaryData: [
                                false,
                            ],
                        },
                    },
                    placeholder: '',
                    description: 'The text content of the file to upload.',
                },
                {
                    displayName: 'Binary Property',
                    name: 'binaryPropertyName',
                    type: 'string',
                    default: 'data',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'upload',
                            ],
                            resource: [
                                'file',
                            ],
                            binaryData: [
                                true,
                            ],
                        },
                    },
                    placeholder: '',
                    description: 'Name of the binary property which contains<br />the data for the file to be uploaded.',
                },
                {
                    displayName: 'Folder',
                    name: 'path',
                    type: 'string',
                    default: '',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'create',
                            ],
                            resource: [
                                'folder',
                            ],
                        },
                    },
                    placeholder: '/invoices/2019',
                    description: 'The folder to create. The parent folder has to exist.',
                },
                {
                    displayName: 'Folder Path',
                    name: 'path',
                    type: 'string',
                    default: '',
                    displayOptions: {
                        show: {
                            operation: [
                                'list',
                            ],
                            resource: [
                                'folder',
                            ],
                        },
                    },
                    placeholder: '/invoices/2019/',
                    description: 'The path of which to list the content.',
                },
            ],
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnData = [];
        const resource = this.getNodeParameter('resource', 0);
        const operation = this.getNodeParameter('operation', 0);
        let endpoint = '';
        let requestMethod = '';
        let body;
        let options;
        const query = {};
        const headers = {};
        for (let i = 0; i < items.length; i++) {
            body = {};
            if (resource === 'file') {
                if (operation === 'download') {
                    requestMethod = 'POST';
                    query.arg = JSON.stringify({
                        path: this.getNodeParameter('path', i),
                    });
                    endpoint = 'https://content.dropboxapi.com/2/files/download';
                }
                else if (operation === 'upload') {
                    requestMethod = 'POST';
                    headers['Content-Type'] = 'application/octet-stream';
                    query.arg = JSON.stringify({
                        mode: 'overwrite',
                        path: this.getNodeParameter('path', i),
                    });
                    endpoint = 'https://content.dropboxapi.com/2/files/upload';
                    if (this.getNodeParameter('binaryData', i) === true) {
                        options = { json: false };
                        const item = items[i];
                        if (item.binary === undefined) {
                            throw new Error('No binary data exists on item!');
                        }
                        const propertyNameUpload = this.getNodeParameter('binaryPropertyName', i);
                        if (item.binary[propertyNameUpload] === undefined) {
                            throw new Error(`No binary data property "${propertyNameUpload}" does not exists on item!`);
                        }
                        body = Buffer.from(item.binary[propertyNameUpload].data, n8n_core_1.BINARY_ENCODING);
                    }
                    else {
                        body = Buffer.from(this.getNodeParameter('fileContent', i), 'utf8');
                    }
                }
            }
            else if (resource === 'folder') {
                if (operation === 'create') {
                    requestMethod = 'POST';
                    body = {
                        path: this.getNodeParameter('path', i),
                    };
                    endpoint = 'https://api.dropboxapi.com/2/files/create_folder_v2';
                }
                else if (operation === 'list') {
                    requestMethod = 'POST';
                    body = {
                        path: this.getNodeParameter('path', i),
                        limit: 2000,
                    };
                    endpoint = 'https://api.dropboxapi.com/2/files/list_folder';
                }
            }
            if (['file', 'folder'].includes(resource)) {
                if (operation === 'copy') {
                    requestMethod = 'POST';
                    body = {
                        from_path: this.getNodeParameter('path', i),
                        to_path: this.getNodeParameter('toPath', i),
                    };
                    endpoint = 'https://api.dropboxapi.com/2/files/copy_v2';
                }
                else if (operation === 'delete') {
                    requestMethod = 'POST';
                    body = {
                        path: this.getNodeParameter('path', i),
                    };
                    endpoint = 'https://api.dropboxapi.com/2/files/delete_v2';
                }
                else if (operation === 'move') {
                    requestMethod = 'POST';
                    body = {
                        from_path: this.getNodeParameter('path', i),
                        to_path: this.getNodeParameter('toPath', i),
                    };
                    endpoint = 'https://api.dropboxapi.com/2/files/move_v2';
                }
            }
            else {
                throw new Error(`The resource "${resource}" is not known!`);
            }
            if (resource === 'file' && operation === 'download') {
                options = { encoding: null };
            }
            let responseData = await GenericFunctions_1.dropboxApiRequest.call(this, requestMethod, endpoint, body, query, headers, options);
            if (resource === 'file' && operation === 'upload') {
                responseData = JSON.parse(responseData);
            }
            if (resource === 'file' && operation === 'download') {
                const newItem = {
                    json: items[i].json,
                    binary: {},
                };
                if (items[i].binary !== undefined) {
                    Object.assign(newItem.binary, items[i].binary);
                }
                items[i] = newItem;
                const dataPropertyNameDownload = this.getNodeParameter('binaryPropertyName', i);
                const filePathDownload = this.getNodeParameter('path', i);
                items[i].binary[dataPropertyNameDownload] = await this.helpers.prepareBinaryData(Buffer.from(responseData), filePathDownload);
            }
            else if (resource === 'folder' && operation === 'list') {
                const propNames = {
                    'id': 'id',
                    'name': 'name',
                    'client_modified': 'lastModifiedClient',
                    'server_modified': 'lastModifiedServer',
                    'rev': 'rev',
                    'size': 'contentSize',
                    '.tag': 'type',
                    'content_hash': 'contentHash',
                };
                for (const item of responseData.entries) {
                    const newItem = {};
                    for (const propName of Object.keys(propNames)) {
                        if (item[propName] !== undefined) {
                            newItem[propNames[propName]] = item[propName];
                        }
                    }
                    returnData.push(newItem);
                }
            }
            else {
                returnData.push(responseData);
            }
        }
        if (resource === 'file' && operation === 'download') {
            return this.prepareOutputData(items);
        }
        else {
            return [this.helpers.returnJsonArray(returnData)];
        }
    }
}
exports.Dropbox = Dropbox;
//# sourceMappingURL=Dropbox.node.js.map