"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RabbitMQ = void 0;
const DefaultOptions_1 = require("./DefaultOptions");
const GenericFunctions_1 = require("./GenericFunctions");
class RabbitMQ {
    constructor() {
        this.description = {
            displayName: 'RabbitMQ',
            name: 'rabbitmq',
            icon: 'file:rabbitmq.png',
            group: ['transform'],
            version: 1,
            description: 'Sends messages to a RabbitMQ topic',
            defaults: {
                name: 'RabbitMQ',
                color: '#ff6600',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'rabbitmq',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Queue / Topic',
                    name: 'queue',
                    type: 'string',
                    default: '',
                    placeholder: 'queue-name',
                    description: 'Name of the queue to publish to.',
                },
                {
                    displayName: 'Send Input Data',
                    name: 'sendInputData',
                    type: 'boolean',
                    default: true,
                    description: 'Send the the data the node receives as JSON to Kafka.',
                },
                {
                    displayName: 'Message',
                    name: 'message',
                    type: 'string',
                    displayOptions: {
                        show: {
                            sendInputData: [
                                false,
                            ],
                        },
                    },
                    default: '',
                    description: 'The message to be sent.',
                },
                {
                    displayName: 'Options',
                    name: 'options',
                    type: 'collection',
                    default: {},
                    placeholder: 'Add Option',
                    options: DefaultOptions_1.rabbitDefaultOptions,
                },
            ],
        };
    }
    async execute() {
        let channel;
        try {
            const items = this.getInputData();
            const queue = this.getNodeParameter('queue', 0);
            const options = this.getNodeParameter('options', 0, {});
            channel = await GenericFunctions_1.rabbitmqConnect.call(this, queue, options);
            const sendInputData = this.getNodeParameter('sendInputData', 0);
            let message;
            const queuePromises = [];
            for (let i = 0; i < items.length; i++) {
                if (sendInputData === true) {
                    message = JSON.stringify(items[i].json);
                }
                else {
                    message = this.getNodeParameter('message', i);
                }
                queuePromises.push(channel.sendToQueue(queue, Buffer.from(message)));
            }
            const promisesResponses = await Promise.allSettled(queuePromises);
            const returnItems = [];
            promisesResponses.forEach((response) => {
                if (response.status !== 'fulfilled') {
                    if (this.continueOnFail() !== true) {
                        throw new Error(response.reason);
                    }
                    else {
                        returnItems.push({
                            json: {
                                error: response.reason,
                            },
                        });
                        return;
                    }
                }
                returnItems.push({
                    json: {
                        success: response.value,
                    },
                });
            });
            await channel.close();
            return this.prepareOutputData(returnItems);
        }
        catch (error) {
            if (channel) {
                await channel.close();
            }
            throw error;
        }
    }
}
exports.RabbitMQ = RabbitMQ;
//# sourceMappingURL=RabbitMQ.node.js.map