"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sortOptions = exports.salesforceApiRequestAllItems = exports.salesforceApiRequest = void 0;
const moment = require("moment-timezone");
const jwt = require("jsonwebtoken");
async function salesforceApiRequest(method, endpoint, body = {}, qs = {}, uri, option = {}) {
    const authenticationMethod = this.getNodeParameter('authentication', 0, 'oAuth2');
    try {
        if (authenticationMethod === 'jwt') {
            const credentialsType = 'salesforceJwtApi';
            const credentials = this.getCredentials(credentialsType);
            const response = await getAccessToken.call(this, credentials);
            const { instance_url, access_token } = response;
            const options = getOptions.call(this, method, (uri || endpoint), body, qs, instance_url);
            options.headers.Authorization = `Bearer ${access_token}`;
            return await this.helpers.request(options);
        }
        else {
            const credentialsType = 'salesforceOAuth2Api';
            const credentials = this.getCredentials(credentialsType);
            const subdomain = (credentials.accessTokenUrl.match(/https:\/\/(.+).salesforce\.com/) || [])[1];
            const options = getOptions.call(this, method, (uri || endpoint), body, qs, `https://${subdomain}.salesforce.com`);
            return await this.helpers.requestOAuth2.call(this, credentialsType, options);
        }
    }
    catch (error) {
        if (error.response && error.response.body && error.response.body[0] && error.response.body[0].message) {
            throw new Error(`Salesforce error response [${error.statusCode}]: ${error.response.body[0].message}`);
        }
        throw error;
    }
}
exports.salesforceApiRequest = salesforceApiRequest;
async function salesforceApiRequestAllItems(propertyName, method, endpoint, body = {}, query = {}) {
    var _a, _b;
    const returnData = [];
    let responseData;
    let uri;
    do {
        responseData = await salesforceApiRequest.call(this, method, endpoint, body, query, uri);
        uri = `${endpoint}/${(_b = (_a = responseData.nextRecordsUrl) === null || _a === void 0 ? void 0 : _a.split('/')) === null || _b === void 0 ? void 0 : _b.pop()}`;
        returnData.push.apply(returnData, responseData[propertyName]);
    } while (responseData.nextRecordsUrl !== undefined &&
        responseData.nextRecordsUrl !== null);
    return returnData;
}
exports.salesforceApiRequestAllItems = salesforceApiRequestAllItems;
function sortOptions(options) {
    options.sort((a, b) => {
        if (a.name < b.name) {
            return -1;
        }
        if (a.name > b.name) {
            return 1;
        }
        return 0;
    });
}
exports.sortOptions = sortOptions;
function getOptions(method, endpoint, body, qs, instanceUrl) {
    const options = {
        headers: {
            'Content-Type': 'application/json',
        },
        method,
        body: method === 'GET' ? undefined : body,
        qs,
        uri: `${instanceUrl}/services/data/v39.0${endpoint}`,
        json: true,
    };
    return options;
}
function getAccessToken(credentials) {
    const now = moment().unix();
    const authUrl = credentials.environment === 'sandbox' ? 'https://test.salesforce.com' : 'https://login.salesforce.com';
    const signature = jwt.sign({
        'iss': credentials.clientId,
        'sub': credentials.username,
        'aud': authUrl,
        'exp': now + 3 * 60,
    }, credentials.privateKey, {
        algorithm: 'RS256',
        header: {
            'alg': 'RS256',
        },
    });
    const options = {
        headers: {
            'Content-Type': 'application/x-www-form-urlencoded',
        },
        method: 'POST',
        form: {
            grant_type: 'urn:ietf:params:oauth:grant-type:jwt-bearer',
            assertion: signature,
        },
        uri: `${authUrl}/services/oauth2/token`,
        json: true,
    };
    return this.helpers.request(options);
}
//# sourceMappingURL=GenericFunctions.js.map