"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Signl4 = void 0;
const n8n_core_1 = require("n8n-core");
const GenericFunctions_1 = require("./GenericFunctions");
class Signl4 {
    constructor() {
        this.description = {
            displayName: 'SIGNL4',
            name: 'signl4',
            icon: 'file:signl4.png',
            group: ['transform'],
            version: 1,
            subtitle: '={{$parameter["operation"] + ": " + $parameter["resource"]}}',
            description: 'Consume SIGNL4 API.',
            defaults: {
                name: 'SIGNL4',
                color: '#53afe8',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'signl4Api',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Resource',
                    name: 'resource',
                    type: 'options',
                    options: [
                        {
                            name: 'Alert',
                            value: 'alert',
                        },
                    ],
                    default: 'alert',
                    description: 'The resource to operate on.',
                },
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    displayOptions: {
                        show: {
                            resource: [
                                'alert',
                            ],
                        },
                    },
                    options: [
                        {
                            name: 'Send',
                            value: 'send',
                            description: 'Send an alert',
                        },
                        {
                            name: 'Resolve',
                            value: 'resolve',
                            description: 'Resolve an alert',
                        },
                    ],
                    default: 'send',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Message',
                    name: 'message',
                    type: 'string',
                    typeOptions: {
                        alwaysOpenEditWindow: true,
                    },
                    default: '',
                    required: false,
                    displayOptions: {
                        show: {
                            operation: [
                                'send',
                            ],
                            resource: [
                                'alert',
                            ],
                        },
                    },
                    description: 'A more detailed description for the alert.',
                },
                {
                    displayName: 'Additional Fields',
                    name: 'additionalFields',
                    type: 'collection',
                    placeholder: 'Add Field',
                    displayOptions: {
                        show: {
                            operation: [
                                'send',
                            ],
                            resource: [
                                'alert',
                            ],
                        },
                    },
                    default: {},
                    options: [
                        {
                            displayName: 'Alerting Scenario',
                            name: 'alertingScenario',
                            type: 'options',
                            options: [
                                {
                                    name: 'Single ACK',
                                    value: 'single_ack',
                                    description: 'In case only one person needs to confirm this Signl.',
                                },
                                {
                                    name: 'Multi ACK',
                                    value: 'multi_ack',
                                    description: 'in case this alert must be confirmed by the number of people who are on duty at the time this Singl is raised',
                                },
                            ],
                            default: 'single_ack',
                            required: false,
                        },
                        {
                            displayName: 'Attachments',
                            name: 'attachmentsUi',
                            placeholder: 'Add Attachments',
                            type: 'fixedCollection',
                            typeOptions: {
                                multipleValues: false,
                            },
                            options: [
                                {
                                    name: 'attachmentsBinary',
                                    displayName: 'Attachments Binary',
                                    values: [
                                        {
                                            displayName: 'Property Name',
                                            name: 'property',
                                            type: 'string',
                                            placeholder: 'data',
                                            default: '',
                                            description: 'Name of the binary properties which contain data which should be added as attachment',
                                        },
                                    ],
                                },
                            ],
                            default: {},
                        },
                        {
                            displayName: 'External ID',
                            name: 'externalId',
                            type: 'string',
                            default: '',
                            description: `If the event originates from a record in a 3rd party system, use this parameter to pass <br/>
						the unique ID of that record. That ID will be communicated in outbound webhook notifications from SIGNL4,<br/>
						which is great for correlation/synchronization of that record with the alert.<br/>
						If you resolve / close an alert you must use the same External ID as in the original alert.`,
                        },
                        {
                            displayName: 'Filtering',
                            name: 'filtering',
                            type: 'boolean',
                            default: 'false',
                            description: `Specify a boolean value of true or false to apply event filtering for this event, or not. <br/>
						If set to true, the event will only trigger a notification to the team, if it contains at least one keyword <br/>
						from one of your services and system categories (i.e. it is whitelisted)`,
                        },
                        {
                            displayName: 'Location',
                            name: 'locationFieldsUi',
                            type: 'fixedCollection',
                            placeholder: 'Add Location',
                            default: {},
                            description: 'Transmit location information (\'latitude, longitude\') with your event and display a map in the mobile app.',
                            options: [
                                {
                                    name: 'locationFieldsValues',
                                    displayName: 'Location',
                                    values: [
                                        {
                                            displayName: 'Latitude',
                                            name: 'latitude',
                                            type: 'string',
                                            required: true,
                                            description: 'The location latitude.',
                                            default: '',
                                        },
                                        {
                                            displayName: 'Longitude',
                                            name: 'longitude',
                                            type: 'string',
                                            required: true,
                                            description: 'The location longitude.',
                                            default: '',
                                        },
                                    ],
                                },
                            ],
                        },
                        {
                            displayName: 'Service',
                            name: 'service',
                            type: 'string',
                            default: '',
                            description: 'Assigns the alert to the service/system category with the specified name.',
                        },
                        {
                            displayName: 'Title',
                            name: 'title',
                            type: 'string',
                            default: '',
                            description: 'The title or subject of this alert.',
                        },
                    ],
                },
                {
                    displayName: 'External ID',
                    name: 'externalId',
                    type: 'string',
                    default: '',
                    required: false,
                    displayOptions: {
                        show: {
                            operation: [
                                'resolve',
                            ],
                            resource: [
                                'alert',
                            ],
                        },
                    },
                    description: `If the event originates from a record in a 3rd party system, use this parameter to pass <br/>
				the unique ID of that record. That ID will be communicated in outbound webhook notifications from SIGNL4,<br/>
				which is great for correlation/synchronization of that record with the alert.<br/>
				If you resolve / close an alert you must use the same External ID as in the original alert.`,
                },
            ],
        };
    }
    async execute() {
        const items = this.getInputData();
        const returnData = [];
        const length = items.length;
        const qs = {};
        let responseData;
        const resource = this.getNodeParameter('resource', 0);
        const operation = this.getNodeParameter('operation', 0);
        for (let i = 0; i < length; i++) {
            if (resource === 'alert') {
                if (operation === 'send') {
                    const message = this.getNodeParameter('message', i);
                    const additionalFields = this.getNodeParameter('additionalFields', i);
                    const data = {
                        message,
                    };
                    if (additionalFields.alertingScenario) {
                        data['X-S4-AlertingScenario'] = additionalFields.alertingScenario;
                    }
                    if (additionalFields.externalId) {
                        data['X-S4-ExternalID'] = additionalFields.externalId;
                    }
                    if (additionalFields.filtering) {
                        data['X-S4-Filtering'] = additionalFields.filtering.toString();
                    }
                    if (additionalFields.locationFieldsUi) {
                        const locationUi = additionalFields.locationFieldsUi.locationFieldsValues;
                        if (locationUi) {
                            data['X-S4-Location'] = `${locationUi.latitude},${locationUi.longitude}`;
                        }
                    }
                    if (additionalFields.service) {
                        data['X-S4-Service'] = additionalFields.service;
                    }
                    data['X-S4-Status'] = 'new';
                    if (additionalFields.title) {
                        data['title'] = additionalFields.title;
                    }
                    const attachments = additionalFields.attachmentsUi;
                    if (attachments) {
                        if (attachments.attachmentsBinary && items[i].binary) {
                            const propertyName = attachments.attachmentsBinary.property;
                            const binaryProperty = items[i].binary[propertyName];
                            if (binaryProperty) {
                                const supportedFileExtension = ['png', 'jpg', 'txt'];
                                if (!supportedFileExtension.includes(binaryProperty.fileExtension)) {
                                    throw new Error(`Invalid extension, just ${supportedFileExtension.join(',')} are supported}`);
                                }
                                data['file'] = {
                                    value: Buffer.from(binaryProperty.data, n8n_core_1.BINARY_ENCODING),
                                    options: {
                                        filename: binaryProperty.fileName,
                                        contentType: binaryProperty.mimeType,
                                    },
                                };
                            }
                            else {
                                throw new Error(`Binary property ${propertyName} does not exist on input`);
                            }
                        }
                    }
                    const credentials = this.getCredentials('signl4Api');
                    const endpoint = `https://connect.signl4.com/webhook/${credentials === null || credentials === void 0 ? void 0 : credentials.teamSecret}`;
                    responseData = await GenericFunctions_1.SIGNL4ApiRequest.call(this, 'POST', '', data, {}, endpoint, {});
                }
                if (operation === 'resolve') {
                    const data = {};
                    data['X-S4-ExternalID'] = this.getNodeParameter('externalId', i);
                    data['X-S4-Status'] = 'resolved';
                    const credentials = this.getCredentials('signl4Api');
                    const endpoint = `https://connect.signl4.com/webhook/${credentials === null || credentials === void 0 ? void 0 : credentials.teamSecret}`;
                    responseData = await GenericFunctions_1.SIGNL4ApiRequest.call(this, 'POST', '', data, {}, endpoint, {});
                }
            }
        }
        if (Array.isArray(responseData)) {
            returnData.push.apply(returnData, responseData);
        }
        else if (responseData !== undefined) {
            returnData.push(responseData);
        }
        return [this.helpers.returnJsonArray(returnData)];
    }
}
exports.Signl4 = Signl4;
//# sourceMappingURL=Signl4.node.js.map