"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Snowflake = void 0;
const GenericFunctions_1 = require("./GenericFunctions");
const snowflake = require("snowflake-sdk");
class Snowflake {
    constructor() {
        this.description = {
            displayName: 'Snowflake',
            name: 'snowflake',
            icon: 'file:snowflake.png',
            group: ['input'],
            version: 1,
            description: 'Get, add and update data in Snowflake.',
            defaults: {
                name: 'Snowflake',
                color: '#5ebbeb',
            },
            inputs: ['main'],
            outputs: ['main'],
            credentials: [
                {
                    name: 'snowflake',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    options: [
                        {
                            name: 'Execute Query',
                            value: 'executeQuery',
                            description: 'Execute an SQL query.',
                        },
                        {
                            name: 'Insert',
                            value: 'insert',
                            description: 'Insert rows in database.',
                        },
                        {
                            name: 'Update',
                            value: 'update',
                            description: 'Update rows in database.',
                        },
                    ],
                    default: 'insert',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Query',
                    name: 'query',
                    type: 'string',
                    typeOptions: {
                        rows: 5,
                    },
                    displayOptions: {
                        show: {
                            operation: [
                                'executeQuery',
                            ],
                        },
                    },
                    default: '',
                    placeholder: 'SELECT id, name FROM product WHERE id < 40',
                    required: true,
                    description: 'The SQL query to execute.',
                },
                {
                    displayName: 'Table',
                    name: 'table',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'insert',
                            ],
                        },
                    },
                    default: '',
                    required: true,
                    description: 'Name of the table in which to insert data to.',
                },
                {
                    displayName: 'Columns',
                    name: 'columns',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'insert',
                            ],
                        },
                    },
                    default: '',
                    placeholder: 'id,name,description',
                    description: 'Comma separated list of the properties which should used as columns for the new rows.',
                },
                {
                    displayName: 'Table',
                    name: 'table',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                        },
                    },
                    default: '',
                    required: true,
                    description: 'Name of the table in which to update data in',
                },
                {
                    displayName: 'Update Key',
                    name: 'updateKey',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                        },
                    },
                    default: 'id',
                    required: true,
                    description: 'Name of the property which decides which rows in the database should be updated. Normally that would be "id".',
                },
                {
                    displayName: 'Columns',
                    name: 'columns',
                    type: 'string',
                    displayOptions: {
                        show: {
                            operation: [
                                'update',
                            ],
                        },
                    },
                    default: '',
                    placeholder: 'name,description',
                    description: 'Comma separated list of the properties which should used as columns for rows to update.',
                },
            ],
        };
    }
    async execute() {
        const credentials = this.getCredentials('snowflake');
        const returnData = [];
        let responseData;
        const connection = snowflake.createConnection(credentials);
        await GenericFunctions_1.connect(connection);
        const items = this.getInputData();
        const operation = this.getNodeParameter('operation', 0);
        if (operation === 'executeQuery') {
            for (let i = 0; i < items.length; i++) {
                const query = this.getNodeParameter('query', i);
                responseData = await GenericFunctions_1.execute(connection, query, []);
                returnData.push.apply(returnData, responseData);
            }
        }
        if (operation === 'insert') {
            const table = this.getNodeParameter('table', 0);
            const columnString = this.getNodeParameter('columns', 0);
            const columns = columnString.split(',').map(column => column.trim());
            const query = `INSERT INTO ${table}(${columns.join(',')}) VALUES (${columns.map(column => '?').join(',')})`;
            const data = GenericFunctions_1.copyInputItems(items, columns);
            const binds = data.map((element => Object.values(element)));
            await GenericFunctions_1.execute(connection, query, binds);
            returnData.push.apply(returnData, data);
        }
        if (operation === 'update') {
            const table = this.getNodeParameter('table', 0);
            const updateKey = this.getNodeParameter('updateKey', 0);
            const columnString = this.getNodeParameter('columns', 0);
            const columns = columnString.split(',').map(column => column.trim());
            if (!columns.includes(updateKey)) {
                columns.unshift(updateKey);
            }
            const query = `UPDATE ${table} SET ${columns.map(column => `${column} = ?`).join(',')} WHERE ${updateKey} = ?;`;
            const data = GenericFunctions_1.copyInputItems(items, columns);
            const binds = data.map((element => Object.values(element).concat(element[updateKey])));
            for (let i = 0; i < binds.length; i++) {
                await GenericFunctions_1.execute(connection, query, binds[i]);
            }
            returnData.push.apply(returnData, data);
        }
        await GenericFunctions_1.destroy(connection);
        return [this.helpers.returnJsonArray(returnData)];
    }
}
exports.Snowflake = Snowflake;
//# sourceMappingURL=Snowflake.node.js.map