"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SpreadsheetFile = void 0;
const n8n_core_1 = require("n8n-core");
const xlsx_1 = require("xlsx");
function flattenObject(data) {
    const returnData = {};
    for (const key1 of Object.keys(data)) {
        if (data[key1] !== null && (typeof data[key1]) === 'object') {
            const flatObject = flattenObject(data[key1]);
            for (const key2 in flatObject) {
                if (flatObject[key2] === undefined) {
                    continue;
                }
                returnData[`${key1}.${key2}`] = flatObject[key2];
            }
        }
        else {
            returnData[key1] = data[key1];
        }
    }
    return returnData;
}
class SpreadsheetFile {
    constructor() {
        this.description = {
            displayName: 'Spreadsheet File',
            name: 'spreadsheetFile',
            icon: 'fa:table',
            group: ['transform'],
            version: 1,
            description: 'Reads and writes data from a spreadsheet file.',
            defaults: {
                name: 'Spreadsheet File',
                color: '#2244FF',
            },
            inputs: ['main'],
            outputs: ['main'],
            properties: [
                {
                    displayName: 'Operation',
                    name: 'operation',
                    type: 'options',
                    options: [
                        {
                            name: 'Read from file',
                            value: 'fromFile',
                            description: 'Reads data from a spreadsheet file',
                        },
                        {
                            name: 'Write to file',
                            value: 'toFile',
                            description: 'Writes the workflow data to a spreadsheet file',
                        },
                    ],
                    default: 'fromFile',
                    description: 'The operation to perform.',
                },
                {
                    displayName: 'Binary Property',
                    name: 'binaryPropertyName',
                    type: 'string',
                    default: 'data',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'fromFile',
                            ],
                        },
                    },
                    placeholder: '',
                    description: 'Name of the binary property from which to read<br />the binary data of the spreadsheet file.',
                },
                {
                    displayName: 'File Format',
                    name: 'fileFormat',
                    type: 'options',
                    options: [
                        {
                            name: 'CSV',
                            value: 'csv',
                            description: 'Comma-separated values',
                        },
                        {
                            name: 'HTML',
                            value: 'html',
                            description: 'HTML Table',
                        },
                        {
                            name: 'ODS',
                            value: 'ods',
                            description: 'OpenDocument Spreadsheet',
                        },
                        {
                            name: 'RTF',
                            value: 'rtf',
                            description: 'Rich Text Format',
                        },
                        {
                            name: 'XLS',
                            value: 'xls',
                            description: 'Excel',
                        },
                        {
                            name: 'XLSX',
                            value: 'xlsx',
                            description: 'Excel',
                        },
                    ],
                    default: 'xls',
                    displayOptions: {
                        show: {
                            operation: [
                                'toFile',
                            ],
                        },
                    },
                    description: 'The format of the file to save the data as.',
                },
                {
                    displayName: 'Binary Property',
                    name: 'binaryPropertyName',
                    type: 'string',
                    default: 'data',
                    required: true,
                    displayOptions: {
                        show: {
                            operation: [
                                'toFile',
                            ],
                        },
                    },
                    placeholder: '',
                    description: 'Name of the binary property in which to save<br />the binary data of the spreadsheet file.',
                },
                {
                    displayName: 'Options',
                    name: 'options',
                    type: 'collection',
                    placeholder: 'Add Option',
                    default: {},
                    options: [
                        {
                            displayName: 'Compression',
                            name: 'compression',
                            type: 'boolean',
                            displayOptions: {
                                show: {
                                    '/operation': [
                                        'toFile',
                                    ],
                                    '/fileFormat': [
                                        'xlsx',
                                        'ods',
                                    ],
                                },
                            },
                            default: false,
                            description: 'Weather compression will be applied or not',
                        },
                        {
                            displayName: 'File Name',
                            name: 'fileName',
                            type: 'string',
                            displayOptions: {
                                show: {
                                    '/operation': [
                                        'toFile',
                                    ],
                                },
                            },
                            default: '',
                            description: 'File name to set in binary data. By default will "spreadsheet.<fileFormat>" be used.',
                        },
                        {
                            displayName: 'Include Empty Cells',
                            name: 'includeEmptyCells',
                            type: 'boolean',
                            displayOptions: {
                                show: {
                                    '/operation': [
                                        'fromFile',
                                    ],
                                },
                            },
                            default: false,
                            description: 'When reading from file the empty cells will be filled with an empty string in the JSON.',
                        },
                        {
                            displayName: 'RAW Data',
                            name: 'rawData',
                            type: 'boolean',
                            displayOptions: {
                                show: {
                                    '/operation': [
                                        'fromFile',
                                    ],
                                },
                            },
                            default: false,
                            description: 'If the data should be returned RAW instead of parsed.',
                        },
                        {
                            displayName: 'Read As String',
                            name: 'readAsString',
                            type: 'boolean',
                            displayOptions: {
                                show: {
                                    '/operation': [
                                        'fromFile',
                                    ],
                                },
                            },
                            default: false,
                            description: 'In some cases and file formats, it is necessary to read<br />specifically as string else some special character get interpreted wrong.',
                        },
                        {
                            displayName: 'Range',
                            name: 'range',
                            type: 'string',
                            displayOptions: {
                                show: {
                                    '/operation': [
                                        'fromFile',
                                    ],
                                },
                            },
                            default: '',
                            description: 'The range to read from the table.<br />If set to a number it will be the starting row.<br />If set to string it will be used as A1-style bounded range.',
                        },
                        {
                            displayName: 'Sheet Name',
                            name: 'sheetName',
                            type: 'string',
                            displayOptions: {
                                show: {
                                    '/operation': [
                                        'fromFile',
                                    ],
                                },
                            },
                            default: 'Sheet',
                            description: 'Name of the sheet to read from in the spreadsheet (if supported). If not set, the first one gets chosen.',
                        },
                        {
                            displayName: 'Sheet Name',
                            name: 'sheetName',
                            type: 'string',
                            displayOptions: {
                                show: {
                                    '/operation': [
                                        'toFile',
                                    ],
                                    '/fileFormat': [
                                        'ods',
                                        'xls',
                                        'xlsx',
                                    ],
                                },
                            },
                            default: 'Sheet',
                            description: 'Name of the sheet to create in the spreadsheet.',
                        },
                    ],
                },
            ],
        };
    }
    async execute() {
        const items = this.getInputData();
        const operation = this.getNodeParameter('operation', 0);
        const newItems = [];
        if (operation === 'fromFile') {
            let item;
            for (let i = 0; i < items.length; i++) {
                item = items[i];
                const binaryPropertyName = this.getNodeParameter('binaryPropertyName', i);
                const options = this.getNodeParameter('options', i, {});
                if (item.binary === undefined || item.binary[binaryPropertyName] === undefined) {
                    continue;
                }
                const binaryData = Buffer.from(item.binary[binaryPropertyName].data, n8n_core_1.BINARY_ENCODING);
                let workbook;
                if (options.readAsString === true) {
                    workbook = xlsx_1.read(binaryData.toString(), { type: 'string', raw: options.rawData });
                }
                else {
                    workbook = xlsx_1.read(binaryData, { raw: options.rawData });
                }
                if (workbook.SheetNames.length === 0) {
                    throw new Error('Spreadsheet does not have any sheets!');
                }
                let sheetName = workbook.SheetNames[0];
                if (options.sheetName) {
                    if (!workbook.SheetNames.includes(options.sheetName)) {
                        throw new Error(`Spreadsheet does not contain sheet called "${options.sheetName}"!`);
                    }
                    sheetName = options.sheetName;
                }
                const sheetToJsonOptions = {};
                if (options.range) {
                    if (isNaN(options.range)) {
                        sheetToJsonOptions.range = options.range;
                    }
                    else {
                        sheetToJsonOptions.range = parseInt(options.range, 10);
                    }
                }
                if (options.includeEmptyCells) {
                    sheetToJsonOptions.defval = '';
                }
                const sheetJson = xlsx_1.utils.sheet_to_json(workbook.Sheets[sheetName], sheetToJsonOptions);
                if (sheetJson.length === 0) {
                    continue;
                }
                for (const rowData of sheetJson) {
                    newItems.push({ json: rowData });
                }
            }
            return this.prepareOutputData(newItems);
        }
        else if (operation === 'toFile') {
            const binaryPropertyName = this.getNodeParameter('binaryPropertyName', 0);
            const fileFormat = this.getNodeParameter('fileFormat', 0);
            const options = this.getNodeParameter('options', 0, {});
            let item;
            const itemData = [];
            for (let itemIndex = 0; itemIndex < items.length; itemIndex++) {
                item = items[itemIndex];
                itemData.push(flattenObject(item.json));
            }
            const ws = xlsx_1.utils.json_to_sheet(itemData);
            const wopts = {
                bookSST: false,
                type: 'buffer',
            };
            if (fileFormat === 'csv') {
                wopts.bookType = 'csv';
            }
            else if (fileFormat === 'html') {
                wopts.bookType = 'html';
            }
            else if (fileFormat === 'rtf') {
                wopts.bookType = 'rtf';
            }
            else if (fileFormat === 'ods') {
                wopts.bookType = 'ods';
                if (options.compression) {
                    wopts.compression = true;
                }
            }
            else if (fileFormat === 'xls') {
                wopts.bookType = 'xls';
            }
            else if (fileFormat === 'xlsx') {
                wopts.bookType = 'xlsx';
                if (options.compression) {
                    wopts.compression = true;
                }
            }
            const sheetName = options.sheetName || 'Sheet';
            const wb = {
                SheetNames: [sheetName],
                Sheets: {
                    [sheetName]: ws,
                },
            };
            const wbout = xlsx_1.write(wb, wopts);
            const newItem = {
                json: {},
                binary: {},
            };
            let fileName = `spreadsheet.${fileFormat}`;
            if (options.fileName !== undefined) {
                fileName = options.fileName;
            }
            newItem.binary[binaryPropertyName] = await this.helpers.prepareBinaryData(wbout, fileName);
            const newItems = [];
            newItems.push(newItem);
            return this.prepareOutputData(newItems);
        }
        else {
            throw new Error(`The operation "${operation}" is not supported!`);
        }
    }
}
exports.SpreadsheetFile = SpreadsheetFile;
//# sourceMappingURL=SpreadsheetFile.node.js.map