"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.uploadAttachments = exports.chunks = exports.twitterApiRequestAllItems = exports.twitterApiRequest = void 0;
const n8n_core_1 = require("n8n-core");
async function twitterApiRequest(method, resource, body = {}, qs = {}, uri, option = {}) {
    let options = {
        method,
        body,
        qs,
        url: uri || `https://api.twitter.com/1.1${resource}`,
        json: true,
    };
    try {
        if (Object.keys(option).length !== 0) {
            options = Object.assign({}, options, option);
        }
        if (Object.keys(body).length === 0) {
            delete options.body;
        }
        if (Object.keys(qs).length === 0) {
            delete options.qs;
        }
        return await this.helpers.requestOAuth1.call(this, 'twitterOAuth1Api', options);
    }
    catch (error) {
        if (error.response && error.response.body && error.response.body.errors) {
            const errorMessages = error.response.body.errors.map((error) => {
                return error.message;
            });
            throw new Error(`Twitter error response [${error.statusCode}]: ${errorMessages.join(' | ')}`);
        }
        throw error;
    }
}
exports.twitterApiRequest = twitterApiRequest;
async function twitterApiRequestAllItems(propertyName, method, endpoint, body = {}, query = {}) {
    const returnData = [];
    let responseData;
    query.count = 100;
    do {
        responseData = await twitterApiRequest.call(this, method, endpoint, body, query);
        query.since_id = responseData.search_metadata.max_id;
        returnData.push.apply(returnData, responseData[propertyName]);
    } while (responseData.search_metadata &&
        responseData.search_metadata.next_results);
    return returnData;
}
exports.twitterApiRequestAllItems = twitterApiRequestAllItems;
function chunks(buffer, chunkSize) {
    const result = [];
    const len = buffer.length;
    let i = 0;
    while (i < len) {
        result.push(buffer.slice(i, i += chunkSize));
    }
    return result;
}
exports.chunks = chunks;
async function uploadAttachments(binaryProperties, items, i) {
    const uploadUri = 'https://upload.twitter.com/1.1/media/upload.json';
    const media = [];
    for (const binaryPropertyName of binaryProperties) {
        const binaryData = items[i].binary;
        if (binaryData === undefined) {
            throw new Error('No binary data set. So file can not be written!');
        }
        if (!binaryData[binaryPropertyName]) {
            continue;
        }
        let attachmentBody = {};
        let response = {};
        const isAnimatedWebp = (Buffer.from(binaryData[binaryPropertyName].data, 'base64').toString().indexOf('ANMF') !== -1);
        const isImage = binaryData[binaryPropertyName].mimeType.includes('image');
        if (isImage && isAnimatedWebp) {
            throw new Error('Animated .webp images are not supported use .gif instead');
        }
        if (isImage) {
            const attachmentBody = {
                media_data: binaryData[binaryPropertyName].data,
            };
            response = await twitterApiRequest.call(this, 'POST', '', {}, {}, uploadUri, { form: attachmentBody });
            media.push(response);
        }
        else {
            attachmentBody = {
                command: 'INIT',
                total_bytes: Buffer.from(binaryData[binaryPropertyName].data, n8n_core_1.BINARY_ENCODING).byteLength,
                media_type: binaryData[binaryPropertyName].mimeType,
            };
            response = await twitterApiRequest.call(this, 'POST', '', {}, {}, uploadUri, { form: attachmentBody });
            const mediaId = response.media_id_string;
            const binaryParts = chunks(Buffer.from(binaryData[binaryPropertyName].data, n8n_core_1.BINARY_ENCODING), 5242880);
            let index = 0;
            for (const binaryPart of binaryParts) {
                attachmentBody = {
                    name: binaryData[binaryPropertyName].fileName,
                    command: 'APPEND',
                    media_id: mediaId,
                    media_data: Buffer.from(binaryPart).toString('base64'),
                    segment_index: index,
                };
                response = await twitterApiRequest.call(this, 'POST', '', {}, {}, uploadUri, { form: attachmentBody });
                index++;
            }
            attachmentBody = {
                command: 'FINALIZE',
                media_id: mediaId,
            };
            response = await twitterApiRequest.call(this, 'POST', '', {}, {}, uploadUri, { form: attachmentBody });
            if (response.processing_info) {
                const { check_after_secs } = response.processing_info;
                await new Promise((resolve, reject) => {
                    setTimeout(() => {
                        resolve();
                    }, check_after_secs * 1000);
                });
            }
            media.push(response);
        }
        return media;
    }
}
exports.uploadAttachments = uploadAttachments;
//# sourceMappingURL=GenericFunctions.js.map